#!/usr/bin/env Rscript 

#This function performs a PCA with an adjusted n_PCs to eliminate random technical noise in the data.
#The choice of the number of PCs to discard is based on the estimates of technical variance in technical, which may be computed by modelGeneVar or modelGeneVarWithSpikes, or deprecated trendVar. 

suppressPackageStartupMessages(require(optparse))
suppressPackageStartupMessages(require(workflowscriptscommon))

# argument parsing 
option_list = list(
  make_option(
    c("-i", "--input-sce-object"),
    action = "store",
    default = NA,
    type = 'character',
    help = 'Path to the input SCE object in rds format.'
  ),
  make_option(
    c("-t", "--technical"),
    action = "store",
    default = NA ,
    type = 'character',
    help = 'This can be: 1) a function that computes the technical component of the variance for a gene as FitTrendVar, 
                         2) a numeric vector of length equal to the number of rows in x, containing the technical component for each gene.  
                         3) a DataFrame of variance decomposition results generated by modelGeneVar or related functions. '
  ),
   make_option(
    c("-s", "--subset_row"),
    action = "store",
    default = NULL,
    type = 'logical',
    help = 'Logical, integer or character vector specifying the rows for which to model the variance. Defaults to all genes in x.'
  ),
  make_option(
    c("-m", "--min-rank"),
    action = "store",
    default = 5,
    type = 'integer',
    help = 'Integer scalars specifying the minimum number of PCs to retain.'
  ),
    make_option(
    c("x", "--max-rank"),
    action = "store",
    default = 50,
    type = 'integer',
    help = 'Integer scalars specifying the maximum number of PCs to retain.'
  ),
  make_option(
    c("-a", "--assay-type"),
    action = "store",
    default = "logcounts",
    type = 'character',
    help = 'String or integer scalar specifying the assay containing the log-expression values.'
  ),
make_option(
    c("-g", "--get-spikes"),
    action = "store",
    default =FALSE,
    type = 'logical',
    help = 'If get-spikes = FALSE, spike-in transcripts are automatically removed. If get.spikes=TRUE, no filtering on the spike-in transcripts will be performed.'
  ),  
make_option(
    c("-v", "--value"),
    action = "store",
    default = "pca",
    type = 'character',
    help = 'String specifying the type of value to return. "pca" will return the PCs, "n" will return the number of retained components, and "lowrank" will return a low-rank approximation.'
  ),
  make_option(
    c("-o", "--output-sce-object"),
    action = "store",
    default = NA,
    type = 'character',
    help = 'Path to the output SCE object with denoised PC number.'
  )
)

opt = wsc_parse_args(option_list, mandatory = c("input_sce_object", "technical", "output_sce_object"))

#read SCE object
if(!file.exists(opt$input_sce_object)) stop("Input file does not exist.")
sce <- readRDS(opt$input_sce_object)

#read technical variance df
if(!file.exists(opt$technical))stop("Object containing the technical components of variation for each gene is not preset, and is required")
variance_trend <- readRDS(opt$technical)
variance_trend_function <- variance_trend$trend

#Compute denoise PCA
suppressPackageStartupMessages(require(scran))
sce <- denoisePCA(sce,technical=variance_trend_function, subset.row=opt$subset_row, min.rank=opt$min_rank,  max.rank=opt$max_rank, assay.type=opt$assay_type, value=opt$value, get.spikes=opt$get_spikes)

#TODO: we may want to include subset.row=getTopHVGs(tech_var, prop=0.1)) for greater accuracy.

#save SCE object with denoised PCA components Factors
saveRDS(sce, opt$output_sce_object)
